#include <stdio.h>
#include <stdlib.h>
#include <float.h>

#include <mef.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

int main(int argc, char **argv) {
  int k = 3;
  int n = 5000;
  int d = 2;
  double *data = malloc(d * k * n * sizeof(double));
  double *x;

  MultivariateGaussian_family* ef = MultivariateGaussian(d);
  
  const gsl_rng_type *T;
  gsl_rng_env_setup();
  T = gsl_rng_default;
  gsl_rng *rng = gsl_rng_alloc(T);

  gsl_vector *mu1 = gsl_vector_alloc(ef->dim);
  gsl_vector_set(mu1, 0, 3);
  gsl_vector_set(mu1, 1, 5);
  gsl_matrix *sigma1 = gsl_matrix_alloc(ef->dim, ef->dim);
  gsl_matrix_set(sigma1, 0, 0, 1.0);
  gsl_matrix_set(sigma1, 1, 0, 0.6);
  gsl_matrix_set(sigma1, 0, 1, 0.6);
  gsl_matrix_set(sigma1, 1, 1, 2.0);
  MultivariateGaussian_source* lambda1 = MultivariateGaussian_create_source(ef, mu1, sigma1);

  gsl_vector *mu2 = gsl_vector_alloc(ef->dim);
  gsl_vector_set(mu2, 0, 0);
  gsl_vector_set(mu2, 1, 0);
  gsl_matrix *sigma2 = gsl_matrix_alloc(ef->dim, ef->dim);
  gsl_matrix_set(sigma2, 0, 0, 1.0);
  gsl_matrix_set(sigma2, 1, 0, 0.6);
  gsl_matrix_set(sigma2, 0, 1, 0.6);
  gsl_matrix_set(sigma2, 1, 1, 2.0);
  MultivariateGaussian_source* lambda2 = MultivariateGaussian_create_source(ef, mu2, sigma2);

  gsl_vector *mu3 = gsl_vector_alloc(ef->dim);
  gsl_vector_set(mu3, 0, -5);
  gsl_vector_set(mu3, 1, -5);
  gsl_matrix *sigma3 = gsl_matrix_alloc(ef->dim, ef->dim);
  gsl_matrix_set(sigma3, 0, 0, 1.0);
  gsl_matrix_set(sigma3, 1, 0, 0.6);
  gsl_matrix_set(sigma3, 0, 1, 0.6);
  gsl_matrix_set(sigma3, 1, 1, 2.0);
  MultivariateGaussian_source* lambda3 = MultivariateGaussian_create_source(ef, mu3, sigma3);

  for (unsigned int i=0; i<n; i++) {
    x = ef->rand(ef, (param*)lambda1, rng);
    data[2*i + 0] = x[0];
    data[2*i + 1] = x[1];
  }
  for (unsigned int i=n; i<2*n; i++) {
    x = ef->rand(ef, (param*)lambda2, rng);
    data[2*i + 0] = x[0];
    data[2*i + 1] = x[1];
  }
  for (unsigned int i=2*n; i<3*n; i++) {
    x = ef->rand(ef, (param*)lambda3, rng);
    data[2*i + 0] = x[0];
    data[2*i + 1] = x[1];
  }

  em* em1 = em_create((family*)ef, k, data, k*n, d);
  /* em1->maxiter  = 100; */
  /* em1->treshold = 1e-5; */

  double best_error = DBL_MAX;
  kmeans* km;
  kmeans* km0;
  for (unsigned int i=0; i<1; i++) {
    km0 = kmeans_create(k, data, k*n, d);
    kmeans_initialize_random(km0, rng);
    kmeans_run(km0);

    if (km0->old_error < best_error) {
      km = km0;
      best_error = km0->old_error;
    }
    else {
      kmeans_destroy(km0);
    }
  }
  for(unsigned int i=0; i<k; i++) {
    printf("km->counts[%i]=%i\n", i, km->counts[i]);
  }

  printf("Initialization:\n");
  em_initialize_from_clustering(em1, km->weights, km->affectation);
  mixture_fprint(stdout, em1->mixture, SOURCE);

  /* printf("\nRunning:\n"); */
  /* mixture mix = em_run(em1); */

  /* /\* for (unsigned int i=0; i<k; i++) { *\/ */
  /* /\*   printf("%i %f ", i, em1->weights[i]); *\/ */
  /* /\*   em1->ef->eta2lambda(em1->ef, em1->components[i], lambda); *\/ */
  /* /\*   em1->ef->fprint(stdout, em1->ef, lambda); *\/ */
  /* /\* } *\/ */

  /* mixture_fprint(stdout, mix, SOURCE); */

  /* em_destroy(em1); */

  /* printf("\nStep by step:\n"); */
  /* /\* Another version, printing information at each step *\/ */
  /* em em2 = em_create((family)ef, k, data, k*n, d); */
  /* em2->maxiter  = 100; */
  /* em2->treshold = 1e-5; */

  /* em_initialize_from_clustering(em2, km->weights, km->affectation); */

  /* for (unsigned int i=0; i<10; i++) { */
  /*   em_step(em2); */
  /*   printf("Step %i, ll=%f\n", em2->iter, em2->ll); */
  /* } */
  /* em_destroy(em2); */

  /* kmeans_destroy(km); */

  /* mixture_fprint(stdout, em2->mixture, SOURCE); */
}

